/*******************************************************************************
    Copyright (c) 2013 NVidia Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Authors: Mark Hairgrove (mhairgrove@nvidia.com)
             John Hubbard   (jhubbard@nvidia.com)
             Subhash Gutti  (sgutti@nvidia.com)

*******************************************************************************/

/*
 * nvidia_uvm_common.h
 *
 * This file contains common header code that can be used for all variants of
 * the UVM kernel module.
 *
 * Initially, UVM-Lite and Uvm-Next work is proceeding somewhat
 * independently, but the long- and short-term plan is for them to share the
 * same kernel module (this one). That's why, at least initially, you'll find
 * files for UVM-Lite, for Uvm-Next, and for common code. We'll refactor as
 * necessary, to achieve the right balance of sharing code and decoupling the
 * features.
 *
 * --jhubbard
 *
 */

#ifndef _NVIDIA_UVM_COMMON_H
#define _NVIDIA_UVM_COMMON_H

#include "uvmtypes.h"
#include "nvidia_uvm_linux.h"

#define NVIDIA_UVM_DEVICE_NAME          "nvidia-uvm"
#define NVIDIA_UVM_NUM_MINOR_DEVICES    2

// Provide a short form of UUID's, typically for use in debug printing:
#define ABBREV_UUID(uuid) (unsigned)(uuid)

static int uvmlite_enabled(void)
{
#ifdef NVIDIA_UVM_LITE_ENABLED
    return 1;
#else
    return 0;
#endif
}

static int uvmnext_enabled(void)
{
#ifdef NVIDIA_UVM_NEXT_ENABLED
    return 1;
#else
    return 0;
#endif
}

static int rm_enabled(void)
{
#ifdef NVIDIA_UVM_RM_ENABLED
    return 1;
#else
    return 0;
#endif
}

//
// Please see the documentation of format_uuid_to_buffer, for details on what
// this routine prints for you.
//
void uvm_print_uuid(const char * function, const char * msg,
                    UvmGpuUuid * pUuidStruct);

static inline void kmem_cache_destroy_safe(struct kmem_cache* cache)
{
    if (cache)
        kmem_cache_destroy(cache);
}

#endif /* _NVIDIA_UVM_COMMON_H */
