/*******************************************************************************
    Copyright (c) 2013 NVidia Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

#ifndef _UVM_GPU_PROVIDER_H
#define _UVM_GPU_PROVIDER_H

#include "uvmtypes.h"
#include "nv_uvm_interface.h"
#include "nvidia_page_migration.h"

#define SEMAPHORE_SIZE    4*1024
#define PUSHBUFFER_SIZE   0x4000

typedef struct UvmGpuProviderTracking_tag
{
    struct UvmGpuProvider *registeredGpuOps;
    struct UvmOpsUvmEvents exportedUvmOps;
  //struct list_head providerList;
}UvmGpuProviderTracking;

UvmGpuProviderTracking* getProvider(void);

//
// These are the UVM datastructures that we need. This should be allocated
// per-process, per-GPU, when setting up a new channel for a process during the
// first call to UvmRegionCommit.
//
typedef struct UvmGpuMigrationTracking_tag
{
    UvmGpuProviderTracking   pProvider;
    uvmGpuSessionHandle      hSession;
    uvmGpuAddressSpaceHandle hVaSpace;
    uvmGpuChannelHandle      hChannel;
    uvmGpuCopyEngineHandle   hCopyEngine;
    unsigned                 ceClassNumber;
    UvmGpuChannelPointers    channelInfo;
    UvmCopyOps               ceOps;

    // per channel allocations
    // pushbuffer
    UvmGpuPointer            gpuPushBufferPtr;
    void                    *cpuPushBufferPtr;
    unsigned                 currentPbOffset;
    unsigned                 currentGpFifoOffset;
    // semaphore
    UvmGpuPointer            gpuSemaPtr;
    void                    *cpuSemaPtr;
    // need to add a lock per push buffer
}UvmGpuMigrationTracking;

int migrate_gpu_to_cpu(UvmGpuMigrationTracking *pMigTracker,
                       UvmGpuPointer srcGpuPtr,
                       UvmGpuPointer destCpuPtr,
                       UvmSize length);

int migrate_cpu_to_gpu(UvmGpuMigrationTracking *pMigTracker,
                       UvmGpuPointer srcCpuPtr,
                       UvmGpuPointer destGpuPtr,
                       UvmSize length);

#endif // _UVM_GPU_PROVIDER_H
