/*******************************************************************************
    Copyright (c) 2013 NVidia Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/
#ifndef _NVIDIA_UVM_LITE_COUNTERS_H_
#define _NVIDIA_UVM_LITE_COUNTERS_H_

#include "nvidia_uvm_lite.h"
#include "uvm-debug.h"

//
// pid = 0 is not a valid value for either the session owner nor the target.
// Therefore it is safe to use 0 for initialization.
//
static const unsigned UVM_PID_INIT_VALUE        =  0;

void UvmIncrementProcessCounters(UvmCounterInfo *procSingleGpuCounter, 
    UvmProcessRecord *pProcessRecord, UvmCounterName name, 
    unsigned incrementVal);

UVM_STATUS NvUvmMapCounter(int sessionIndex, UvmCounterScope scope,
    UvmCounterName counterName, UvmGpuUuid gpuUuid, 
    UvmProcessRecord *pCurrentProcessRecord, void ** addr);

UVM_STATUS UvmMapPage(struct page *page, void * mappingBaseAddress);

//
// Function declarations for counter support:
//
void NvUvmInitSessionInfoArray(UvmProcessRecord *pCurrentProcessRecord);


void UvmInitSessionInfo(UvmSessionInfo *pSessionInfo);

UVM_STATUS NvUvmAddSessionInfo(unsigned pidTarget, uid_t euidTarget, 
                               int *pSessionIndex,
                               UvmProcessCounterInfo *pProcessCounterInfo,
                               unsigned long long *mappedUserBaseAddress,
                               UvmProcessRecord *pCurrentProcessRecord);

UVM_STATUS NvUvmRemoveSessionInfo(int sessionIndex,
                                  UvmProcessRecord *pCurrentProcessRecord);

void NvUvmFreeCounters(UvmCounterInfo *ctrInfo);

UVM_STATUS NvUvmCounterStateUpdate(int sessionIndex,
    const UvmCounterConfig *config, unsigned count,
    UvmProcessRecord *pCurrentProcessRecord);

UVM_STATUS NvUvmIncrementSessionCount(UvmCounterInfo *pCtrInfo,
    UvmCounterName counterName);
UVM_STATUS NvUvmDecrementSessionCount(UvmCounterInfo *pCtrInfo,
    UvmCounterName counterName);

UVM_STATUS NvUvmGetCounterIndex(UvmCounterName counterName,
    unsigned *counterIndex);

UvmBool UvmSecurityCheck(unsigned pidTarget);

unsigned UvmGetProcessId(void);

UVM_STATUS NvUvmGetSessionCounterInfoSecure(int sessionIndex,
    UvmProcessRecord *pCurrentProcessRecord,
    UvmProcessCounterInfo* *pProcessCounterInfo);

#endif // _NVIDIA_UVM_LITE_COUNTERS_H_
